@@ -19,17 +19,19 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "system.h"
+#include "paths.h"
 
 #include <stdio.h>
 #include <cmpbuf.h>
 #include <c-stack.h>
 #include <error.h>
 #include <exitfail.h>
-#include <freesoft.h>
+#include <file-type.h>
 #include <getopt.h>
 #include <hard-locale.h>
 #include <inttostr.h>
 #include <setmode.h>
+#include <version-etc.h>
 #include <xalloc.h>
 #include <xstrtol.h>
 
@@ -39,13 +41,11 @@
 # define hard_locale_LC_MESSAGES 0
 #endif
 
-static char const authorship_msgid[] =
-  N_("Written by Torbjorn Granlund and David MacKenzie.");
-
-static char const copyright_string[] =
-  "Copyright (C) 2002 Free Software Foundation, Inc.";
-
-extern char const version_string[];
+/* TRANSLATORS: Please translate the second "o" in "Torbjorn Granlund"
+   to an o-with-umlaut (U+00F6, LATIN SMALL LETTER O WITH DIAERESIS)
+   if possible.  */
+static char const authorship_msgid[] = N_("\
+Written by Torbjorn Granlund and David MacKenzie.");
 
 static int cmp (void);
 static off_t file_position (int);
@@ -200,7 +200,7 @@ main (int argc, char **argv)
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
-  c_stack_action (c_stack_die);
+  c_stack_action (argv, 0);
 
   /* Parse command line options.  */
 
@@ -239,9 +239,7 @@ main (int argc, char **argv)
 	break;
 
       case 'v':
-	printf ("cmp %s\n%s\n\n%s\n\n%s\n",
-		version_string, copyright_string,
-		_(free_software_msgid), _(authorship_msgid));
+	version_etc ("cmp", authorship_msgid);
 	check_stdout ();
 	return EXIT_SUCCESS;
 
@@ -479,9 +477,7 @@ cmp (void)
 		    bool use_byte_message = (byte_message != byte_msgid
 					     || hard_locale_LC_MESSAGES);
 
-		    printf ((use_byte_message
-			     ? byte_message
-			     : "%s %s differ: char %s, line %s\n"),
+		    printf (use_byte_message ? byte_message : char_message,
 			    file[0], file[1], byte_num, line_num);
 		  }
 		else
